/****************************************************************************
** Meta object code from reading C++ file 'bmdbusinterface.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "backend/bmdbusinterface.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'bmdbusinterface.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15BMDBusInterfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto BMDBusInterface::qt_create_metaobjectdata<qt_meta_tag_ZN15BMDBusInterfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "BMDBusInterface",
        "DeviceListChanged",
        "",
        "addlistJson",
        "dellistJson",
        "Finished",
        "errcode",
        "description",
        "ReportProgress",
        "current",
        "total",
        "title",
        "ReportProgress1",
        "DeviceList",
        "QDBusPendingReply<QString>",
        "Install",
        "QDBusPendingReply<bool>",
        "image",
        "device",
        "partition",
        "formatDevice",
        "CheckFile",
        "filepath",
        "Reboot",
        "QDBusPendingReply<>",
        "Start",
        "Stop"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'DeviceListChanged'
        QtMocHelpers::SignalData<void(const QString &, const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 4 },
        }}),
        // Signal 'Finished'
        QtMocHelpers::SignalData<void(int, const QString &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 6 }, { QMetaType::QString, 7 },
        }}),
        // Signal 'ReportProgress'
        QtMocHelpers::SignalData<void(int, int, const QString &, const QString &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 9 }, { QMetaType::Int, 10 }, { QMetaType::QString, 11 }, { QMetaType::QString, 7 },
        }}),
        // Signal 'ReportProgress1'
        QtMocHelpers::SignalData<void(int, int, const QString &, const QString &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 9 }, { QMetaType::Int, 10 }, { QMetaType::QString, 11 }, { QMetaType::QString, 7 },
        }}),
        // Slot 'DeviceList'
        QtMocHelpers::SlotData<QDBusPendingReply<QString>()>(13, 2, QMC::AccessPublic, 0x80000000 | 14),
        // Slot 'Install'
        QtMocHelpers::SlotData<QDBusPendingReply<bool>(const QString &, const QString &, const QString &, bool)>(15, 2, QMC::AccessPublic, 0x80000000 | 16, {{
            { QMetaType::QString, 17 }, { QMetaType::QString, 18 }, { QMetaType::QString, 19 }, { QMetaType::Bool, 20 },
        }}),
        // Slot 'CheckFile'
        QtMocHelpers::SlotData<QDBusPendingReply<bool>(const QString &)>(21, 2, QMC::AccessPublic, 0x80000000 | 16, {{
            { QMetaType::QString, 22 },
        }}),
        // Slot 'Reboot'
        QtMocHelpers::SlotData<QDBusPendingReply<>()>(23, 2, QMC::AccessPublic, 0x80000000 | 24),
        // Slot 'Start'
        QtMocHelpers::SlotData<QDBusPendingReply<>()>(25, 2, QMC::AccessPublic, 0x80000000 | 24),
        // Slot 'Stop'
        QtMocHelpers::SlotData<QDBusPendingReply<>()>(26, 2, QMC::AccessPublic, 0x80000000 | 24),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<BMDBusInterface, qt_meta_tag_ZN15BMDBusInterfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject BMDBusInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractInterface::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15BMDBusInterfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15BMDBusInterfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15BMDBusInterfaceE_t>.metaTypes,
    nullptr
} };

void BMDBusInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<BMDBusInterface *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->DeviceListChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 1: _t->Finished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 2: _t->ReportProgress((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4]))); break;
        case 3: _t->ReportProgress1((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4]))); break;
        case 4: { QDBusPendingReply<QString> _r = _t->DeviceList();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QString>*>(_a[0]) = std::move(_r); }  break;
        case 5: { QDBusPendingReply<bool> _r = _t->Install((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<bool>*>(_a[0]) = std::move(_r); }  break;
        case 6: { QDBusPendingReply<bool> _r = _t->CheckFile((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<bool>*>(_a[0]) = std::move(_r); }  break;
        case 7: { QDBusPendingReply<> _r = _t->Reboot();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 8: { QDBusPendingReply<> _r = _t->Start();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 9: { QDBusPendingReply<> _r = _t->Stop();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (BMDBusInterface::*)(const QString & , const QString & )>(_a, &BMDBusInterface::DeviceListChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (BMDBusInterface::*)(int , const QString & )>(_a, &BMDBusInterface::Finished, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (BMDBusInterface::*)(int , int , const QString & , const QString & )>(_a, &BMDBusInterface::ReportProgress, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (BMDBusInterface::*)(int , int , const QString & , const QString & )>(_a, &BMDBusInterface::ReportProgress1, 3))
            return;
    }
}

const QMetaObject *BMDBusInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *BMDBusInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15BMDBusInterfaceE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractInterface::qt_metacast(_clname);
}

int BMDBusInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractInterface::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 10;
    }
    return _id;
}

// SIGNAL 0
void BMDBusInterface::DeviceListChanged(const QString & _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void BMDBusInterface::Finished(int _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void BMDBusInterface::ReportProgress(int _t1, int _t2, const QString & _t3, const QString & _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2, _t3, _t4);
}

// SIGNAL 3
void BMDBusInterface::ReportProgress1(int _t1, int _t2, const QString & _t3, const QString & _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2, _t3, _t4);
}
QT_WARNING_POP
