/****************************************************************************
** Meta object code from reading C++ file 'bootmakerservice.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "bootmakerservice.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'bootmakerservice.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN16BootMakerServiceE_t {};
} // unnamed namespace

template <> constexpr inline auto BootMakerService::qt_create_metaobjectdata<qt_meta_tag_ZN16BootMakerServiceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "BootMakerService",
        "D-Bus Interface",
        "com.deepin.bootmaker",
        "DeviceListChanged",
        "",
        "addlistJson",
        "dellistJson",
        "Finished",
        "errcode",
        "description",
        "ReportProgress",
        "current",
        "error",
        "title",
        "ReportProgress1",
        "s_StartBootMarker",
        "Reboot",
        "Start",
        "Stop",
        "DeviceList",
        "Install",
        "image",
        "device",
        "partition",
        "formatDevice",
        "CheckFile",
        "filepath"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'DeviceListChanged'
        QtMocHelpers::SignalData<void(const QString, const QString &)>(3, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::QString, 5 }, { QMetaType::QString, 6 },
        }}),
        // Signal 'Finished'
        QtMocHelpers::SignalData<void(int, const QString &)>(7, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::Int, 8 }, { QMetaType::QString, 9 },
        }}),
        // Signal 'ReportProgress'
        QtMocHelpers::SignalData<void(int, int, const QString &, const QString &)>(10, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::Int, 11 }, { QMetaType::Int, 12 }, { QMetaType::QString, 13 }, { QMetaType::QString, 9 },
        }}),
        // Signal 'ReportProgress1'
        QtMocHelpers::SignalData<void(int, int, const QString &, const QString &)>(14, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void, {{
            { QMetaType::Int, 11 }, { QMetaType::Int, 12 }, { QMetaType::QString, 13 }, { QMetaType::QString, 9 },
        }}),
        // Signal 's_StartBootMarker'
        QtMocHelpers::SignalData<void()>(15, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'Reboot'
        QtMocHelpers::SlotData<void()>(16, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void),
        // Slot 'Start'
        QtMocHelpers::SlotData<void()>(17, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void),
        // Slot 'Stop'
        QtMocHelpers::SlotData<void()>(18, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Void),
        // Slot 'DeviceList'
        QtMocHelpers::SlotData<QString()>(19, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::QString),
        // Slot 'Install'
        QtMocHelpers::SlotData<bool(const QString &, const QString &, const QString &, bool)>(20, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Bool, {{
            { QMetaType::QString, 21 }, { QMetaType::QString, 22 }, { QMetaType::QString, 23 }, { QMetaType::Bool, 24 },
        }}),
        // Slot 'CheckFile'
        QtMocHelpers::SlotData<bool(const QString &)>(25, 4, QMC::AccessPublic | QMC::MethodScriptable, QMetaType::Bool, {{
            { QMetaType::QString, 26 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<BootMakerService, qt_meta_tag_ZN16BootMakerServiceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject BootMakerService::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16BootMakerServiceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16BootMakerServiceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16BootMakerServiceE_t>.metaTypes,
    nullptr
} };

void BootMakerService::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<BootMakerService *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->DeviceListChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 1: _t->Finished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 2: _t->ReportProgress((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4]))); break;
        case 3: _t->ReportProgress1((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4]))); break;
        case 4: _t->s_StartBootMarker(); break;
        case 5: _t->Reboot(); break;
        case 6: _t->Start(); break;
        case 7: _t->Stop(); break;
        case 8: { QString _r = _t->DeviceList();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 9: { bool _r = _t->Install((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 10: { bool _r = _t->CheckFile((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (BootMakerService::*)(const QString , const QString & )>(_a, &BootMakerService::DeviceListChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (BootMakerService::*)(int , const QString & )>(_a, &BootMakerService::Finished, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (BootMakerService::*)(int , int , const QString & , const QString & )>(_a, &BootMakerService::ReportProgress, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (BootMakerService::*)(int , int , const QString & , const QString & )>(_a, &BootMakerService::ReportProgress1, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (BootMakerService::*)()>(_a, &BootMakerService::s_StartBootMarker, 4))
            return;
    }
}

const QMetaObject *BootMakerService::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *BootMakerService::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16BootMakerServiceE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QDBusContext"))
        return static_cast< QDBusContext*>(this);
    return QObject::qt_metacast(_clname);
}

int BootMakerService::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void BootMakerService::DeviceListChanged(const QString _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void BootMakerService::Finished(int _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void BootMakerService::ReportProgress(int _t1, int _t2, const QString & _t3, const QString & _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2, _t3, _t4);
}

// SIGNAL 3
void BootMakerService::ReportProgress1(int _t1, int _t2, const QString & _t3, const QString & _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2, _t3, _t4);
}

// SIGNAL 4
void BootMakerService::s_StartBootMarker()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}
QT_WARNING_POP
