/****************************************************************************
** Meta object code from reading C++ file 'bminterface.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "bminterface.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'bminterface.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN11BMInterfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto BMInterface::qt_create_metaobjectdata<qt_meta_tag_ZN11BMInterfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "BMInterface",
        "deviceListChanged",
        "",
        "QList<DeviceInfo>",
        "addlist",
        "dellist",
        "finished",
        "errcode",
        "description",
        "reportProgress",
        "current",
        "total",
        "title",
        "reportProgress1",
        "startInstall",
        "image",
        "device",
        "partition",
        "formatDevice",
        "startInstallRet",
        "success",
        "start",
        "stop",
        "reboot",
        "deviceList",
        "checkfile",
        "filepath"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'deviceListChanged'
        QtMocHelpers::SignalData<void(const QList<DeviceInfo> &, const QList<DeviceInfo> &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 3, 5 },
        }}),
        // Signal 'finished'
        QtMocHelpers::SignalData<void(int, const QString &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 7 }, { QMetaType::QString, 8 },
        }}),
        // Signal 'reportProgress'
        QtMocHelpers::SignalData<void(int, int, const QString &, const QString &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 10 }, { QMetaType::Int, 11 }, { QMetaType::QString, 12 }, { QMetaType::QString, 8 },
        }}),
        // Signal 'reportProgress1'
        QtMocHelpers::SignalData<void(int, int, const QString &, const QString &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 10 }, { QMetaType::Int, 11 }, { QMetaType::QString, 12 }, { QMetaType::QString, 8 },
        }}),
        // Signal 'startInstall'
        QtMocHelpers::SignalData<void(const QString &, const QString &, const QString &, bool)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 15 }, { QMetaType::QString, 16 }, { QMetaType::QString, 17 }, { QMetaType::Bool, 18 },
        }}),
        // Signal 'startInstallRet'
        QtMocHelpers::SignalData<void(bool)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 20 },
        }}),
        // Slot 'start'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'stop'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'reboot'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'deviceList'
        QtMocHelpers::SlotData<QList<DeviceInfo>()>(24, 2, QMC::AccessPublic, 0x80000000 | 3),
        // Slot 'checkfile'
        QtMocHelpers::SlotData<bool(const QString &)>(25, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 26 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<BMInterface, qt_meta_tag_ZN11BMInterfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject BMInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11BMInterfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11BMInterfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11BMInterfaceE_t>.metaTypes,
    nullptr
} };

void BMInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<BMInterface *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->deviceListChanged((*reinterpret_cast<std::add_pointer_t<QList<DeviceInfo>>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<DeviceInfo>>>(_a[2]))); break;
        case 1: _t->finished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 2: _t->reportProgress((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4]))); break;
        case 3: _t->reportProgress1((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4]))); break;
        case 4: _t->startInstall((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4]))); break;
        case 5: _t->startInstallRet((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 6: _t->start(); break;
        case 7: _t->stop(); break;
        case 8: _t->reboot(); break;
        case 9: { QList<DeviceInfo> _r = _t->deviceList();
            if (_a[0]) *reinterpret_cast<QList<DeviceInfo>*>(_a[0]) = std::move(_r); }  break;
        case 10: { bool _r = _t->checkfile((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<DeviceInfo> >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (BMInterface::*)(const QList<DeviceInfo> & , const QList<DeviceInfo> & )>(_a, &BMInterface::deviceListChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (BMInterface::*)(int , const QString & )>(_a, &BMInterface::finished, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (BMInterface::*)(int , int , const QString & , const QString & )>(_a, &BMInterface::reportProgress, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (BMInterface::*)(int , int , const QString & , const QString & )>(_a, &BMInterface::reportProgress1, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (BMInterface::*)(const QString & , const QString & , const QString & , bool )>(_a, &BMInterface::startInstall, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (BMInterface::*)(bool )>(_a, &BMInterface::startInstallRet, 5))
            return;
    }
}

const QMetaObject *BMInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *BMInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11BMInterfaceE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Dtk::Core::DSingleton<BMInterface>"))
        return static_cast< Dtk::Core::DSingleton<BMInterface>*>(this);
    return QObject::qt_metacast(_clname);
}

int BMInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void BMInterface::deviceListChanged(const QList<DeviceInfo> & _t1, const QList<DeviceInfo> & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void BMInterface::finished(int _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void BMInterface::reportProgress(int _t1, int _t2, const QString & _t3, const QString & _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2, _t3, _t4);
}

// SIGNAL 3
void BMInterface::reportProgress1(int _t1, int _t2, const QString & _t3, const QString & _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2, _t3, _t4);
}

// SIGNAL 4
void BMInterface::startInstall(const QString & _t1, const QString & _t2, const QString & _t3, bool _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1, _t2, _t3, _t4);
}

// SIGNAL 5
void BMInterface::startInstallRet(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}
QT_WARNING_POP
